import unittest

from test_common import mavenreq, assertIn


class TestMavenReq(unittest.TestCase):

    #@mavenreq(["invalid.xml"])
    #def test_invalid(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0)

    #@mavenreq(["not_xml.xml"])
    #def test_not_xml(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0)

    #@mavenreq(["nonexistent_file_blablabla"])
    #def test_nonexistent(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0)

    @mavenreq(["require1/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "mvn(org.apache.maven:maven-project)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require-java/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_java(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "mvn(org.apache.maven:maven-project)",
                "java-headless >= 1:1.6")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require-java2/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_java2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "mvn(org.apache.maven:maven-project)",
                "java-headless >= 1:1.8")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require-java3/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_java3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "mvn(org.apache.maven:maven-project)",
                "java-headless >= 1:1.8")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require-java-fail/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_java_fail(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')
        self.assertEqual(True, "ValueError: Unknown Java version 1.4.2" in stderr)

    @mavenreq(["require_parent/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_parent(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "ns-mvn(org.codehaus.plexus:plexus-ant-factory)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_multi(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "ns-mvn(org.codehaus.plexus:plexus-ant-factory)",
                "ns-mvn(codehaus:plexus-utils) = 1.2", "java-headless",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi_namespaces/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_multi_namespace(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "ns2-mvn(codehaus:plexus-cipher)", "ns-mvn(codehaus:plexus-utils)",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi_versioned/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_multi_versioned(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("java-headless", "javapackages-tools",
                "ns-mvn(org.codehaus.plexus:plexus-ant-factory:1.0) = 1.0",
                "ns-mvn(codehaus:plexus-utils:1.2)",
                "ns-mvn(codehaus:plexus-cipher:1.0) = 1.1",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)",
                "mvn(org.apache.maven.plugins:maven-idea-plugin:1.5) = 1.4")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_mixed/buildroot/usr/share/maven-metadata/require.xml"])
    def test_mixed(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "ns-mvn(org.codehaus.plexus:plexus-ant-factory)",
                "ns-mvn(codehaus:plexus-utils) = 1.2",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)",
                "mvn(org.apache.maven.plugins:maven-idea-plugin)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["requires_simple_subpackages/buildroot/usr/share/maven-metadata/require.xml"])
    def test_simple_subpackage(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "mvn(org.apache.maven:maven-plugin-api) = 3.2.1")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["requires_simple_subpackages2/buildroot/usr/share/maven-metadata/require.xml"])
    def test_simple_subpackage2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "mvn(org.apache.maven:maven-plugin-api) = 3.2.1",
                "mvn(org.codehaus.plexus:plexus-utils)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["requires_simple_subpackages2_compat/buildroot/usr/share/maven-metadata/require.xml"])
    def test_simple_subpackage3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                 "mvn(org.apache.maven:maven-plugin-api:3.2.0) = 3.2.1")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["requires_simple_subpackages2_compat2/buildroot/usr/share/maven-metadata/require.xml"])
    def test_simple_subpackage4(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "mvn(org.apache.maven:maven-plugin-api)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["requires_on_artifact_from_same_package/buildroot/usr/share/maven-metadata/require.xml"])
    def test_simple_artifact_in_same_package(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless")
        self.assertEqual(set(want), set(sout))

    #test for rhbz#1012980
    @mavenreq(["require_skipped/buildroot/usr/share/maven-metadata/require.xml"])
    def test_require_skipped(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertNotEqual(stderr, '')
        lines = stderr.split('\n')
        self.assertEqual(True, len(lines) > 1)
        self.assertEqual("g:skipped:UNKNOWN required by g:a1", lines[-5])
        assertIn(self, "%mvn_package", lines[-3])

    # rhbz#1017701 comment 2
    @mavenreq(["aether/buildroot/usr/share/maven-metadata/require.xml"])
    def test_rhbz1017701_c2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless",
                "maven31-mvn(org.eclipse.aether:aether-api) = 0.9.0.M3")
        self.assertEqual(set(want), set(sout))


    @mavenreq(["require1/buildroot/usr/share/maven-metadata/require.xml"], javaconfdirs=['alternative-java'])
    def test_java_config(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-spineless",
                "mvn(org.apache.maven:maven-project)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi/buildroot/usr/share/maven-metadata/require.xml"], javaconfdirs=['filtered'])
    def test_dep_filtering(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("ns-mvn(org.codehaus.plexus:plexus-ant-factory)", "java-headless",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)",
                "javapackages-tools")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi/buildroot/usr/share/maven-metadata/require.xml"], javaconfdirs=['filtered/'])
    def test_config_env1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("ns-mvn(org.codehaus.plexus:plexus-ant-factory)", "java-headless",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)",
                "javapackages-tools")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_multi/buildroot/usr/share/maven-metadata/require.xml"],
              javaconfdirs=['filtered/', 'alternative-java/'])
    def test_config_env2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("ns-mvn(org.codehaus.plexus:plexus-ant-factory)", "java-headless",
                "mvn(org.apache.maven.wagon:wagon-provider-api::test-jar:)",
                "javapackages-tools")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require2/buildroot/usr/share/maven-metadata/require.xml"],
              xmvnresolve_output="pom_deps")
    def test_deps_from_pom(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(org.fedoraproject.xmvn:xmvn-core)",
                "java-headless", "javapackages-tools",
                "mvn(org.fedoraproject.xmvn:xmvn-api)")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require3/buildroot/usr/share/maven-metadata/require.xml"],
              xmvnresolve_output="pom_parent_deps")
    def test_deps_from_pom_with_parent(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(plugin:external)", "javapackages-tools",
                "mvn(ppom:parent-pom:pom:)", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require4/buildroot/usr/share/maven-metadata/require.xml"],
              xmvnresolve_output="subpkg_deps")
    def test_deps_from_pom_on_subpkg(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(extension:from-subpackage) = 1.1", "javapackages-tools",
                "mvn(ppom:parent-pom:pom:)", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require5/buildroot/usr/share/maven-metadata/require.xml"])
    def test_unknown_dep(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        serr = [x for x in stderr.split('\n') if x]
        want = ("org.apache.maven:maven-project:2.2.1")
        assertIn(self, want, serr)

    @mavenreq(["require6/buildroot/usr/share/maven-metadata/require.xml"],
              xmvnresolve_output="pom_deps_fail")
    def test_pom_dep_fail(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        want = ("unresolvable:pom-dependency:pom:2.2.1")
        self.assertTrue(want in stderr, stderr)

    @mavenreq(["osgi_simple/buildroot/usr/share/maven-metadata/require.xml"])
    def test_osgi_basic(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("osgi(osgi.req1)", "javapackages-tools", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["osgi_self/buildroot/usr/share/maven-metadata/require.xml"])
    def test_osgi_self(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("javapackages-tools", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["osgi_versioned/buildroot/usr/share/maven-metadata/require.xml"])
    def test_osgi_versioned(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("osgi(osgi.req1) = 1.0", "javapackages-tools", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["osgi_versioned_ns/buildroot/usr/share/maven-metadata/require.xml"])
    def test_osgi_versioned_ns(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("devtoolset-3-osgi(osgi.req1) = 1.0", "javapackages-tools", "java-headless")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["dashes/buildroot/usr/share/maven-metadata/require.xml"])
    def test_dashes(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(org.apache.maven:maven-plugin-api) = 1.alpha.2",
                "osgi(osgi2) = 1.5.1.SNAPSHOT",
                "java-headless", "javapackages-tools")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require_optional/buildroot/usr/share/maven-metadata/require.xml"])
    def test_optional(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(dg:da)", "java-headless", "javapackages-tools")
        self.assertEqual(set(want), set(sout))

    @mavenreq(["require-no-artifacts/buildroot/usr/share/maven-metadata/require.xml"])
    def test_no_artifacts(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        sout = [x for x in stdout.split('\n') if x]
        want = ("java-headless", "javapackages-tools")
        self.assertEqual(set(want), set(sout))

if __name__ == '__main__':
    unittest.main()



import inspect
import os
import unittest
import shutil
from test_common import *
from lxml import etree
from xml_compare import compare_lxml_etree


class TestMvnArtifact(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        dirpath = os.path.dirname(os.path.realpath(__file__))
        cls.datadir = os.path.join(dirpath, 'data', 'mvn_artifact')

        try:
            cls.olddir = os.getcwd()
            cls.workdir = os.path.join(dirpath, 'data', 'artifact_workdir')

            shutil.copytree(cls.datadir, cls.workdir)
            os.chdir(cls.workdir)
        except OSError:
            pass

        prepare_metadata(cls.workdir)

    @classmethod
    def tearDownClass(cls):
        try:
            shutil.rmtree(cls.workdir)
            os.chdir(cls.olddir)
        except OSError:
            pass

    def tearDown(self):
        try:
            os.remove('.xmvn-reactor')
        except OSError:
            pass

    def xml_compare_reporter(self, report):
        print(report)

    def check_result(self, test_name):
        got = etree.parse(".xmvn-reactor").getroot()
        want = etree.parse(os.path.join(self.workdir,
                                        test_name+"-want.xml")).getroot()
        report = compare_lxml_etree(got, want, unordered=['dependencies'])
        if report:
            report = '\n' + report
        return report

    @mvn_artifact('args4j.pom')
    def test_basic(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('args4j.pom', 'maven-artifact.jar')
    def test_basic_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('args4j.pom', 'webapp.war')
    def test_artifact_war(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('args4j.ivy')
    def test_ivy(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('args4j.ivy', 'maven-artifact.jar')
    def test_ivy_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('args4j.ivy', 'webapp.war')
    def test_ivy_war(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:12', 'test.jar')
    def test_mvn_spec_jar(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:war:12', 'test.war')
    def test_mvn_spec_war(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:jar:12', 'test.jar')
    def test_mvn_spec_ext(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:war:12', 'test.war')
    def test_mvn_spec_ext_war(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b::sources:12', 'test-sources.jar')
    def test_mvn_spec_classifier(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:war:javadoc:12', 'test-javadoc.war')
    def test_mvn_spec_classifier_war(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    @mvn_artifact('a:b:12')
    def test_mvn_spec_nojar(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)

    @mvn_artifact('a:b:tar:javadoc:12', 'test-javadoc.war')
    def test_extensions_dont_match(self, stdout, stderr, return_value):
        # different extensions
        self.assertNotEqual(return_value, 0, stderr)

    @mvn_artifact('a:b:jar:javadoc:12', 'test-javadoc.war')
    def test_extensions_dont_match2(self, stdout, stderr, return_value):
        # jar != war
        self.assertNotEqual(return_value, 0, stderr)

    @mvn_artifact('a:b::javadoc:12', 'test-javadoc.jar')
    def test_extension_not_specified(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)

    #@mvn_artifact('args4j-deps.pom')
    #def test_dependencies_1(self, stdout, stderr, return_value):
    #    self.assertEqual(return_value, 0, stderr)
    #    report = self.check_result(inspect.currentframe().f_code.co_name)
    #    self.assertEqual(report, '', report)

    #@mvn_artifact('args4j-deps.pom', 'some.jar')
    #def test_dependencies_2(self, stdout, stderr, return_value):
    #    self.assertEqual(return_value, 0, stderr)
    #    report = self.check_result(inspect.currentframe().f_code.co_name)
    #    self.assertEqual(report, '', report)

    #@mvn_artifact('args4j-exclusions.pom', 'some.jar')
    #def test_dependencies_exclusions(self, stdout, stderr, return_value):
    #    self.assertEqual(return_value, 0, stderr)
    #    report = self.check_result(inspect.currentframe().f_code.co_name)
    #    self.assertEqual(report, '', report)

    #@mvn_artifact('asm-analysis-5.0.2.pom')
    #def test_missing_version(self, stdout, stderr, return_value):
    #    self.assertNotEqual(return_value, 0, stderr)

    @mvn_artifact('merge_sections/child/child/pom.xml')
    def test_merge_sections(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

    # fedora-java/javapackages#3
    @mvn_artifact('test_parent.pom.xml')
    def test_parent(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        report = self.check_result(inspect.currentframe().f_code.co_name)
        self.assertEqual(report, '', report)

if __name__ == '__main__':
    unittest.main()



from test_common import exec_pom_macro_simple, WorkdirTestCase, assertIn
import unittest

class TestGeneric(WorkdirTestCase):
    def test_inject(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_inject "/project/target[@name='init']" '
                       <mkdir dir="lib"/>
                ' build.xml""",
                'build.xml', 'build_with_lib.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_remove(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_remove "/project/target[@name='dist']" build.xml""",
                'build.xml', 'build_without_dist.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_set(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_set "target[@name='clean']" '<echo/>' build.xml""",
                'build.xml', 'build_set.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_set_attribute(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_set "jar/@jarfile" '${dist}/other.jar' build.xml""",
                'build.xml', 'build_attr.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_remove_attribute(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_remove "jar/@basedir" build.xml""",
                'build.xml', 'build_attr_removed.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_namespace(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_set "target/ivy:cachepath/@type" 'war' build.xml""",
                'build_ns.xml', 'build_ns_set.xml', filename='build.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)



class TestIvy(WorkdirTestCase):
    def test_inject(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_xpath_inject dependencies '
                       <dependency org="foo" name="bar" transitive="false"/>
                '""",
                'simple_ivy.xml', 'simple_ivy_added_foo_bar.xml', filename='ivy.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_unsupported(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_parent",
                "simple_ivy.xml", filename='ivy.xml')
        assertIn(self, 'Operation not supported', stderr)
        self.assertNotEqual(0, return_value)
        self.assertEqual('', report)

    def test_add_dep(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                """%pom_add_dep 'org.eclipse.jetty.orbit:javax.servlet:3.1:servlet->default:false' '
                    <artifact name="javax.servlet" type="orbit" ext="jar"/>
                '""",
                'simple_ivy.xml', 'simple_ivy_added_orbit.xml', filename='ivy.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_remove_dep(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_remove_dep 'xerces:'",
                'simple_ivy.xml', 'simple_ivy_removed_xerces.xml', filename='ivy.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_change_dep(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep ':xercesImpl' ':xerces'",
                'simple_ivy.xml', 'simple_ivy_xerces_aid.xml', filename='ivy.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

    def test_remove_rev(self):
        return_value, stderr, report = exec_pom_macro_simple(\
                "%pom_change_dep ':nekohtml' '::-'",
                'simple_ivy.xml', 'simple_ivy_remove_rev.xml', filename='ivy.xml')
        self.assertEqual(0, return_value, stderr)
        self.assertEqual('', report, report)

if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from lxml.etree import ElementTree, XMLParser
from javapackages.common.exception import JavaPackagesToolsException


class PomLoadingException(JavaPackagesToolsException):
    pass


POM_NAMESPACE = "http://maven.apache.org/POM/4.0.0"


def load(pom_path):
    et = ElementTree()
    parser = XMLParser(remove_comments=True, strip_cdata=True)
    try:
        doc = et.parse(pom_path, parser=parser)
    except IOError:
        raise PomLoadingException("Cannot read file {f}".format(f=pom_path))

    if doc is None:
        raise PomLoadingException("Failed to load {f}".format(f=pom_path))
    return doc


def find(doc, xpath_str, namespace=POM_NAMESPACE):
    ret = xpath(doc, xpath_str, namespace)
    if len(ret) > 0:
        ret = ret[0]
    else:
        ret = None
    return ret


def xpath(doc, xpath_str, namespace=POM_NAMESPACE):
    ret = doc.xpath(xpath_str, namespaces=dict(pom=namespace))
    # perhaps there is no namespace?
    if len(ret) == 0:
        ret = doc.xpath(xpath_str.replace('pom:', ''))
    return ret


def find_parts(doc, parts, xpath_str=".//"):
    for key in parts:
        node = doc.xpath('{0}*[local-name() = "{1}"]'.format(xpath_str, key))
        if node is not None and len(node) > 0 and node[0].text is not None:
            parts[key] = node[0].text.strip()
    return parts


def find_raw_parts(doc, parts, xpath_str=".//"):
    for key in parts:
        node = doc.xpath('{0}*[local-name() = "{1}"]'.format(xpath_str, key))
        if node is not None and len(node) > 0:
            if node[0].text is not None:
                parts[key] = node[0].text.strip()
            else:
                # node is present, but it has no content
                parts[key] = ""
        else:
            parts[key] = None
    return parts



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

from javapackages.maven.artifact import Artifact
import javapackages.common.strutils as Printer
from javapackages.metadata.exclusion import MetadataExclusion

from javapackages.common.binding import ObjectBinding

import six


class MetadataDependency(ObjectBinding):
    element_name = 'dependency'
    fields = ['groupId', 'artifactId', 'extension', 'classifier',
              'namespace', 'optional', 'requestedVersion',
              'resolvedVersion', 'exclusions']
    defaults = {'extension': 'jar',
                'requestedVersion': 'SYSTEM'}
    types = {'optional': str,  # todo bool
             'exclusions': set([MetadataExclusion])}

    def is_optional(self):
        if self.optional and self.optional.lower() == "true":
            return True
        return False

    def get_mvn_str(self):
        return Printer.get_mvn_str(self.groupId,
                                   self.artifactId,
                                   ext=self.extension,
                                   cla=self.classifier,
                                   ver=self.requestedVersion)

    def get_rpm_str(self, namespace=None, compat_ver=None, pkg_ver=None):
        return Printer.get_rpm_str(self.groupId,
                                   self.artifactId,
                                   ext=self.extension,
                                   cla=self.classifier,
                                   namespace=namespace or self.namespace,
                                   compat_ver=compat_ver or self.resolvedVersion,
                                   pkg_ver=pkg_ver)

    def is_provided_by(self, artifacts):
        for provided in artifacts:
            if (provided.groupId == self.groupId and
               provided.artifactId == self.artifactId and
               provided.classifier == self.classifier and
               provided.extension == self.extension and
               provided.namespace == self.namespace):

                if self.resolvedVersion and provided.is_compat():
                    # does it match one of provided compat?
                    for compatVer in provided.compatVersions:
                        if self.resolvedVersion == compatVer:
                            return True, provided.version
                elif (not self.resolvedVersion and
                      not provided.is_compat()):
                    return True, provided.version
                break
        return False, None

    def is_skipped(self, skipped_artifacts):
        for skipped in skipped_artifacts:
            if (self.groupId == skipped.groupId and
               self.artifactId == skipped.artifactId and
               self.classifier == skipped.classifier and
               self.extension == skipped.extension):
                return True
        return False

    def __unicode__(self):
        return six.text_type(self.get_mvn_str())

    def __str__(self):
        return self.__unicode__()

    def __hash__(self):
        h = 47
        h += 12 + hash(self.groupId)
        h += 22 + hash(self.artifactId)
        h += 32 + hash(self.extension)
        h += 42 + hash(self.classifier)
        h += 62 + hash(self.namespace)
        h += 72 + hash(self.resolvedVersion)
        return h

    def __eq__(self, other):
        if type(other) is not type(self):
            return False
        if (self.groupId == other.groupId and
           self.artifactId == other.artifactId and
           self.extension == other.extension and
           self.classifier == other.classifier and
           self.namespace == other.namespace and
           self.resolvedVersion == other.resolvedVersion):
            return True
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    @classmethod
    def from_mvn_str(cls, mvn_str):
        a = Artifact.from_mvn_str(mvn_str)

        return cls(a.groupId, a.artifactId, extension=a.extension,
                   classifier=a.classifier, requestedVersion=a.version)

    @classmethod
    def from_mvn_dependency(cls, mvn_dep):
        exclusions = set()
        for e in mvn_dep.exclusions:
            exclusions.add(MetadataExclusion.from_mvn_exclusion(e))

        return cls(mvn_dep.groupId, mvn_dep.artifactId,
                   extension=mvn_dep.extension,
                   classifier=mvn_dep.classifier,
                   optional=mvn_dep.optional,
                   requestedVersion=mvn_dep.version,
                   exclusions=exclusions)



#
# Copyright (c) 2015, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import javapackages.common.config as config
from javapackages.common.osgi import OSGiBundle
from javapackages.maven.artifact import Artifact
from javapackages.maven.pom import POM
import javapackages.common.strutils as Printer

from javapackages.metadata.alias import MetadataAlias
from javapackages.metadata.dependency import MetadataDependency

from javapackages.common.binding import ObjectBinding

import six
import os


class MetadataArtifact(ObjectBinding):
    element_name = 'artifact'
    fields = ['groupId', 'artifactId', 'extension', 'classifier',
              'version', 'namespace', 'path', 'aliases', 'properties',
              'compatVersions', 'dependencies']
    defaults = {'extension': 'jar'}
    types = {'compatVersions': set(['version']),
             'aliases': set([MetadataAlias]),
             'dependencies': set([MetadataDependency]),
             'properties': dict}

    def __init__(self, *args, **kwargs):
        super(MetadataArtifact, self).__init__(*args, **kwargs)
        for alias in self.aliases:
            if 'extension' not in alias and 'extension' in self:
                alias.extension = self.extension

    def is_compat(self):
        """Return true if artifact has compat verions specified.
        This means package should have versioned provides for this artifact"""
        return True if self.compatVersions else False

    def has_osgi_information(self):
        return "osgi.id" in self.properties

    def get_osgi_bundle(self):
        if not self.properties:
            return None
        return OSGiBundle.from_properties(self.properties)

    def get_buildroot_path(self, prefix=None):
        if not self.path:
            return None
        if prefix is None:
            prefix = config.get_buildroot()
        return os.path.join(prefix, self.path[1:])

    def get_mvn_str(self):
        return Printer.get_mvn_str(self.groupId,
                                   self.artifactId,
                                   ext=self.extension,
                                   cla=self.classifier,
                                   ver=self.version)

    def get_rpm_str(self, namespace=None, pkg_ver=None):
        result = []

        if not self.is_compat():
            # non-compat rpm string for main artifact
            result.append(Printer.get_rpm_str(self.groupId,
                                              self.artifactId,
                                              ext=self.extension,
                                              cla=self.classifier,
                                              namespace=namespace,
                                              pkg_ver=pkg_ver))

            # non-compat rpm string(s) for aliases
            for alias in self.aliases:
                result.append(Printer.get_rpm_str(alias.groupId,
                                                  alias.artifactId,
                                                  ext=alias.extension,
                                                  cla=alias.classifier,
                                                  namespace=namespace,
                                                  pkg_ver=pkg_ver))
        else:
            # compat rpm string(s) for main artifact
            for compat_ver in self.compatVersions:
                result.append(Printer.get_rpm_str(self.groupId,
                                                  self.artifactId,
                                                  ext=self.extension,
                                                  cla=self.classifier,
                                                  compat_ver=compat_ver,
                                                  namespace=namespace,
                                                  pkg_ver=pkg_ver))

                # compat rpm string(s) for aliases
                for alias in self.aliases:
                    result.append(Printer.get_rpm_str(alias.groupId,
                                                      alias.artifactId,
                                                      ext=alias.extension,
                                                      cla=alias.classifier,
                                                      compat_ver=compat_ver,
                                                      namespace=namespace,
                                                      pkg_ver=pkg_ver))
        return "\n".join(result)

    def __unicode__(self):
        return six.text_type(self.get_mvn_str())

    def __str__(self):
        return self.__unicode__()

    def __hash__(self):
        h = 26
        h += 11 + hash(self.groupId)
        h += 21 + hash(self.artifactId)
        h += 31 + hash(self.extension)
        h += 41 + hash(self.classifier)
        h += 61 + hash(self.namespace)
        h += 71 + hash(self.is_compat())
        return h

    def __eq__(self, other):
        if type(other) is not type(self):
            return False
        if (self.groupId == other.groupId and
           self.artifactId == other.artifactId and
           self.extension == other.extension and
           self.classifier == other.classifier and
           self.namespace == other.namespace and
           self.is_compat() == other.is_compat()):
            return True
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    @classmethod
    def from_pom(cls, pom_path):
        pom = POM(pom_path)
        return cls(pom.groupId, pom.artifactId, version=pom.version,
                   path=pom_path)

    @classmethod
    def from_mvn_str(cls, mvn_str):
        a = Artifact.from_mvn_str(mvn_str)

        return cls(a.groupId, a.artifactId, version=a.version,
                   extension=a.extension, classifier=a.classifier)



import unittest
from test_common import bindir_script


class TestAbs2Rel(unittest.TestCase):

    @bindir_script('abs2rel', ['dummy'])
    def test_usage(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 1)
        self.assertEqual('', stdout)
        self.assertEqual('Usage: abs2rel <PATH> <BASE>\n', stderr)

    @bindir_script('abs2rel', ['/1/2/3/a/b/c', '/1/2/3'])
    def test_abs_abs(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0)
        self.assertEqual('a/b/c\n', stdout)
        self.assertEqual('', stderr)

    @bindir_script('abs2rel', ['foo/bar', 'foo/baz'])
    def test_rel_rel(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0)
        self.assertEqual('../bar\n', stdout)
        self.assertEqual('', stderr)


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>


import sys
import nose

# run nose
if __name__ == "__main__":

    exclude = ["--exclude"]
    # skip these tests
    #exclude.append("maven_req_test.py")

    args = ['foo']     # nose ignores the first argument
    # generate xunit report, if nose version is at least 1.x.x
    if nose.__versioninfo__[0] > 0:
        args.append("--with-xunit")

    if len(exclude) > 1:
        args += exclude

    success = nose.run(argv=args)
    if not success:
        sys.exit(1)



#
# Copyright (c) 2014, Red Hat, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors: Stanislav Ochotnicky <sochotnicky@redhat.com>
#
# this script is used by add_maven_depmap rpm macro to generate
# mapping between maven groupId:artifactId and jar file in our local
# filesystem (i.e. %{_javadir})
# rpm macro expects to find this file as %{_javadir}-utils/maven_depmap.py


from optparse import OptionParser
import os
import shutil
import sys

from os.path import basename, dirname
import zipfile
from time import gmtime, strftime
from copy import deepcopy

from javapackages.maven.pom import POM
from javapackages.metadata.artifact import MetadataArtifact
from javapackages.metadata.alias import MetadataAlias
from javapackages.metadata.metadata import Metadata

from javapackages.common.exception import JavaPackagesToolsException


class PackagingTypeMissingFile(JavaPackagesToolsException):
    def __init__(self, pom_path):
        self.args=("Packaging type is not 'pom' and no artifact path has been provided for POM %s" % pom_path,)

class IncompatibleFilenames(JavaPackagesToolsException):
    def __init__(self, pom_path, jar_path):
        self.args=("Filenames of POM %s and JAR %s does not match properly. Check that JAR subdirectories matches '.' in pom name." % (pom_path, jar_path),)

class ExtensionsDontMatch(JavaPackagesToolsException):
    def __init__(self, coordinates_ext, file_ext):
        self.args=("Extensions don't match: '%s' != '%s'" % (coordinates_ext, file_ext),)

class MissingJarFile(JavaPackagesToolsException):
    def __init__(self):
        self.args=("JAR seems to be missing in standard directories. Make sure you have installed it",)

class UnknownFileExtension(JavaPackagesToolsException):
    def __init__(self, jar_path):
        self.args=("Unknown file extension: %s" % (jar_path),)


def _print_path_with_dirs(path, base):
    print(path)
    path = dirname(path)
    while path != base and path != '/':
        print("%dir " + path)
        path = dirname(path)


def _make_files_versioned(versions, pom_path, jar_path, pom_base, jar_base):
    """Make pom and jar file versioned"""
    versions = list(set(versions.split(',')))

    vpom_path = pom_path
    vjar_path = jar_path

    ret_pom_path = pom_path
    ret_jar_path = jar_path

    # pom
    if ':' not in vpom_path:
        root, ext = os.path.splitext(vpom_path)
        symlink = False
        for ver in sorted(versions):
            dest = "%s-%s%s" % (root, ver, ext)
            if not symlink:
                shutil.copy(os.path.realpath(vpom_path), dest)
                symlink = True
                vpom_path = dest
                ret_pom_path = dest
            else:
                os.symlink(basename(vpom_path), dest)
            # output file path for file lists
            _print_path_with_dirs(dest, pom_base)
        # remove unversioned pom
        os.remove(pom_path)

    # jar
    if vjar_path:
        root, ext = os.path.splitext(vjar_path)
        symlink = False
        for ver in sorted(versions):
            dest = "%s-%s%s" % (root, ver, ext)
            if not symlink:
                shutil.copy(os.path.realpath(vjar_path), dest)
                symlink = True
                vjar_path = dest
                ret_jar_path = dest
            else:
                os.symlink(basename(vjar_path), dest)
            # output file path for file lists
            _print_path_with_dirs(dest, jar_base)
        # remove unversioned jar
        os.remove(jar_path)

    # return paths to versioned, but regular files (not symlinks)
    return ret_pom_path, ret_jar_path


# Add a file to a ZIP archive (or JAR, WAR, ...) unless the file
# already exists in the archive.  Provided by Tomas Radej.
def append_if_missing(archive_name, file_name, file_contents):
    archive = zipfile.ZipFile(archive_name, 'a')
    try:
        if file_name not in archive.namelist():
            archive.writestr(file_name, file_contents)
    finally:
        archive.close()


# Inject pom.properties if JAR doesn't have one.  This is necessary to
# identify the origin of JAR files that are present in the repository.
def inject_pom_properties(jar_path, artifact):
    if not zipfile.is_zipfile(jar_path):
        return
    props_path = "META-INF/maven/{a.groupId}/{a.artifactId}/pom.properties".format(a=artifact)
    properties = """#Generated by Java Packages Tools
version={a.version}
groupId={a.groupId}
artifactId={a.artifactId}
""".format(a=artifact)

    if artifact.extension:
        properties = properties + \
            "extension={ext}\n".format(ext=artifact.extension)
    if artifact.classifier:
        properties = properties + \
            "classifier={clas}\n".format(clas=artifact.classifier)

    append_if_missing(jar_path, props_path, properties)


def add_compat_versions(artifact, versions):
    if not versions:
        return artifact

    artifact.compatVersions = versions.split(',')
    return artifact


def add_aliases(artifact, additions):
    if not additions:
        return artifact

    aliases = additions.split(',')
    result = set()
    for a in aliases:
        alias = MetadataAlias.from_mvn_str(a)
        alias.extension = artifact.extension
        result.add(alias)

    artifact.aliases = result
    return artifact


def write_metadata(metadata_file, artifacts):
    if os.path.exists(metadata_file):
        metadata = Metadata.create_from_file(metadata_file)
    else:
        metadata = Metadata()

    # pylint:disable=E1103
    metadata.artifacts += deepcopy(artifacts)

    metadata.write_to_file(metadata_file)


def _main():
    usage="usage: %prog [options] metadata_path pom_path|<MVN spec> [jar_path]"
    parser = OptionParser(usage=usage)
    parser.add_option("-a","--append",type="str",
                      help="Additional depmaps to add (gid:aid)  [default: %default]")
    parser.add_option('-r', '--versions', type="str",
                      help='Additional versions to add for each depmap')
    parser.add_option('-n', '--namespace', type="str",
                      help='Namespace to use for generated fragments', default="")
    parser.add_option('--pom-base', type="str",
                      help='Base path under which POM files are installed', default="")
    parser.add_option('--jar-base', type="str",
                      help='Base path under which JAR files are installed', default="")

    parser.set_defaults(append=None)

    (options, args) = parser.parse_args()
    append_deps = options.append
    add_versions = options.versions
    namespace = options.namespace
    pom_base = options.pom_base
    jar_base = options.jar_base

    if len(args) < 2:
        parser.error("Incorrect number of arguments")
    # These will fail when incorrect number of arguments is given.
    metadata_path = args[0].strip()
    pom_path = args[1].strip()
    jar_path = None

    artifact = None
    have_pom = False

    if len(args) == 3:
        jar_path = args[2].strip()
        if ':' in pom_path:
            pom_str = pom_path.rsplit('/')[-1]
            artifact = MetadataArtifact.from_mvn_str(pom_str)
            artifact_ext = artifact.extension or "jar"
            file_ext = os.path.splitext(jar_path)[1][1:]
            if artifact_ext != file_ext:
                raise ExtensionsDontMatch(artifact_ext, file_ext)

            if artifact.extension == 'jar':
                artifact.extension = ''

            if not artifact.version:
                parser.error("Artifact definition has to include version")
        else:
            artifact = MetadataArtifact.from_pom(pom_path)
            ext = os.path.splitext(jar_path)[1][1:]
            if ext != "jar":
                artifact.extension = ext
            have_pom = True
        if artifact:
            inject_pom_properties(jar_path, artifact)
    else:
        # looks like POM only artifact
        if ':' not in pom_path:
            artifact = MetadataArtifact.from_pom(pom_path)
            have_pom = True

            if POM(pom_path).packaging != "pom":
                raise PackagingTypeMissingFile(pom_path)
        else:
            sys.exit("JAR file path must be specified when using artifact coordinates")


    # output file path for file lists
    print(metadata_path)

    artifact = add_compat_versions(artifact, add_versions)
    if add_versions:
        pom_path, jar_path = _make_files_versioned(add_versions, pom_path, jar_path, pom_base, jar_base)

    if namespace:
        artifact.namespace = namespace

    artifact.properties["xmvn.resolver.disableEffectivePom"] = "true"


    buildroot = os.environ.get('RPM_BUILD_ROOT')
    am = []
    if jar_path:
        metadata_jar_path = os.path.abspath(jar_path)
        artifact.path = metadata_jar_path.replace(buildroot, "") if buildroot else metadata_jar_path
        artifact = add_aliases(artifact, append_deps)
        if artifact.extension == "jar":
            artifact.extension = ""
        am.append(artifact.copy())
        # output file path for file list (if it's not versioned)
        if not add_versions:
            _print_path_with_dirs(jar_path, jar_base)
    if have_pom:
        metadata_pom_path = os.path.abspath(pom_path)
        artifact.path = metadata_pom_path.replace(buildroot, "") if buildroot else metadata_pom_path
        artifact.extension = "pom"
        artifact.aliases = None
        artifact = add_aliases(artifact, append_deps)
        am.append(artifact.copy())
        # output file path for file list (if it's not versioned)
        if not add_versions:
            _print_path_with_dirs(pom_path, pom_base)

    write_metadata(metadata_path, am)


if __name__ == "__main__":
    try:
        _main()
    except JavaPackagesToolsException as e:
        sys.exit(e)



import io
import os
import re
import sys
import subprocess
import shutil

from textwrap import dedent

DIRPATH = os.path.dirname(os.path.realpath(__file__))

class Package(object):
    """Represents single RPM package that should be built.

    It creates a temporary specfile that is used to build the package using
    rpmbuild.  Every output goes to directory 'rpmbuild' (will be created) in
    current working directory.  RPM macros in javapackages/etc are processed to
    override default lookup for our srcipts in order to test the current
    version and not the system-wide javapackages-tools

    """
    def __init__(self, name):
        self.__name = name
        self.__sources = []
        self.__begin = ''
        self.__prep = ''
        self.__build = ''
        self.__install = ''
        self.buildpath = os.path.join('rpmbuild', 'BUILD', name + '-1')
        self.__env = dict(os.environ)
        self.__env['HOME'] = os.getcwd()
        self.__env['PATH'] = '{mock}:{path}'.format(mock=DIRPATH,
                                            path=self.__env['PATH'])

    def add_source(self, sourcepath, newname=None):
        """Add source file to a package

        File will be copied into 'rpmbuild/SOURCES' and 'rpmbuild/BUILD' by
        default. Path can be absolute or relative to test directory.

        Keyword arguments:
        newname -- name for source file. If None, sourcepath's basename will be
        used.

        """
        if not os.path.isabs(sourcepath):
            sourcepath = os.path.join(DIRPATH, sourcepath)
        if not newname:
            newname = os.path.basename(sourcepath)
        self.__sources.append((sourcepath, newname))

    def prepend(self, to_prepend):
        """Prepends given string to spec file"""
        self.__begin += to_prepend

    def append_to_prep(self, to_append):
        """Appends given string to %prep section of spec."""
        self.__prep += to_append + '\n'

    def append_to_build(self, to_append):
        """Appends given string to %build section of spec."""
        self.__build += to_append + '\n'

    def append_to_install(self, to_append):
        """Appends given string to %install section of spec."""
        self.__install += to_append + '\n'

    def run_prep(self, args=None):
        """Runs rpmbuild -bp (doing only prep phase) with current settings."""
        self.__prepare_all()
        return self.__invoke_rpmbuild(['-bp'] + (args or []))

    def run_build(self, args=None):
        """Runs rpmbuild -bc (stop after build phase) with current settings."""
        self.__prepare_all()
        return self.__invoke_rpmbuild(['-bc'] + (args or []))

    def run_install(self, args=None):
        """Runs rpmbuild -bi (stop after install) with current settings."""
        self.__prepare_all()
        return self.__invoke_rpmbuild(['-bi'] + (args or []))

    def __invoke_rpmbuild(self, args):
        outfile = open("tmpout", 'w')
        errfile = open("tmperr", 'w')
        topdir = '--define=_topdir {cwd}/rpmbuild'.format(cwd=os.getcwd())
        scl_undef = '--eval=%undefine scl'
        proc = subprocess.Popen(['rpmbuild', topdir, scl_undef,
                                 os.path.join('rpmbuild', 'SPECS',
                                              self.__name + '.spec')]
                                + args, shell=False, env=self.__env,
                                stdout=outfile, stderr=errfile)
        ret = proc.wait()
        outfile = open("tmpout", 'r+')
        errfile = open("tmperr", 'r+')
        out = outfile.read()
        err = errfile.read()
        os.remove('tmpout')
        os.remove('tmperr')
        return (out, err, ret)

    def set_env(self, name, value):
        self.__env[name] = value

    def __prepare_all(self):
        try:
            os.mkdir('rpmbuild')
            os.mkdir('rpmbuild/SOURCES')
            os.mkdir('rpmbuild/SPECS')
            os.mkdir('rpmbuild/RPMS')
            os.mkdir('rpmbuild/SRPMS')
            os.mkdir('rpmbuild/BUILD')
            os.mkdir('rpmbuild/BUILDROOT')
        except OSError:
            pass
        _prepare_macros()
        self.__prepare_spec()
        self.__prepare_sources()

    def __prepare_spec(self):
        header = self.__begin + dedent("""
        Name:    {name}
        Version:    1
        Release:    1%{{?dist}}
        Summary:    test
        License:    GPL
        URL:        www.example.com
        """.format(name=self.__name))
        for index, (_, filename) in enumerate(self.__sources):
            filename = os.path.basename(filename)
            header += 'Source{index}: {filename}{index}\n'.format(filename=filename,
                    index=index)

        header += dedent("""\
        %description
        This is just a test.
        """)

        prep_section = dedent("""\

        %prep
        mkdir %{name}-%{version}
        cd ./%{name}-%{version}
        """)
        for index, (_, filename) in enumerate(self.__sources):
            directory = os.path.dirname(filename)
            prep_section += 'mkdir -p ./{d}\n'.format(d=directory)
            prep_section += 'cp -p %SOURCE{index} ./{d}/{f}\n'.format(index=index,
                d=directory, f=os.path.basename(filename))

        prep_section += self.__prep

        build_section = dedent("""\

        %build
        cd ./%{name}-%{version}
        """)
        build_section += self.__build

        install_section = dedent("""\

        %install
        """)
        install_section += self.__install

        specpath = os.path.join('rpmbuild', 'SPECS', self.__name + '.spec')
        with open(specpath, 'w') as specfile:
            specfile.write(header)
            specfile.write(prep_section)
            specfile.write(build_section)
            specfile.write(install_section)

    def __prepare_sources(self):
        destpath = os.path.join('rpmbuild', 'SOURCES')
        for index, (sourcepath, targetname) in enumerate(self.__sources):
            target = os.path.join(destpath, os.path.basename(targetname))
            shutil.copy(sourcepath, '{0}{1}'.format(target, index))


def _prepare_macros():
    macropath = os.path.join(DIRPATH, '..', 'macros.d')
    java_utils = os.path.abspath(os.path.join(DIRPATH, '..', 'java-utils'))

    with io.open('.rpmmacros', 'wt', encoding='utf-8') as rpmmacros:
        for filepath in os.listdir(macropath):
            if filepath.startswith('macros'):
                with io.open(os.path.join(macropath, filepath), encoding='utf-8') as macrofile:
                    for line in macrofile.readlines():
                        if '/usr/share/java-utils' in line:
                            line = re.sub(r'/usr/share/java-utils',
                                          java_utils, line)
                        if '@{javadir}-utils' in line:
                            line = re.sub(r'@\{javadir\}-utils',
                                          java_utils, line)
                        if '@{pyinterpreter}' in line:
                            line = re.sub(r'@\{pyinterpreter\}', sys.executable, line)
                        rpmmacros.write(line)



import unittest
import socket

from test_common import *


class PmRequestTest(unittest.TestCase):

    def setUp(self):
        self.sock_path = "/tmp/test_pm_request_socket"
        self.env = {'PM_REQUEST_SOCKET': self.sock_path}

        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.sock.bind(self.sock_path)
        self.sock.listen(1)

        self.scriptpath = path.join(DIRPATH, '..', 'java-utils', 'request-artifact.py')

    def test_ok(self):
        (out, err, proc) = call_script(self.scriptpath,
                                       ['maven', 'junit:junit'],
                                       extra_env=self.env, async=True)
        conn, _ = self.sock.accept()
        request = conn.makefile().readline().rstrip()
        self.assertEqual(request, "install 'mvn(junit:junit)'")
        conn.sendall(b"ok\n")
        conn.close()
        ret = proc.wait()
        self.assertEqual(ret, 0)

    def test_nok(self):
        (out, err, proc) = call_script(self.scriptpath,
                                       ['maven', 'junit:junit'],
                                       extra_env=self.env, async=True)
        conn, _ = self.sock.accept()
        request = conn.makefile().readline().rstrip()
        self.assertEqual(request, "install 'mvn(junit:junit)'")
        conn.sendall(b"nok\n")
        conn.close()
        ret = proc.wait()
        self.assertEqual(ret, 2)

    def test_jar(self):
        (out, err, proc) = call_script(self.scriptpath,
                                       ['maven', 'gid:aid:jar::'],
                                       extra_env=self.env, async=True)
        conn, _ = self.sock.accept()
        request = conn.makefile().readline().rstrip()
        self.assertEqual(request, "install 'mvn(gid:aid)'")
        conn.sendall(b"ok\n")
        conn.close()
        ret = proc.wait()
        self.assertEqual(ret, 0)

    def test_full_coords(self):
        (out, err, proc) = call_script(self.scriptpath,
                                       ['maven', 'gid:aid:ext:cla:ver'],
                                       extra_env=self.env, async=True)
        conn, _ = self.sock.accept()
        request = conn.makefile().readline().rstrip()
        self.assertEqual(request, "install 'mvn(gid:aid:ext:cla:ver)'")
        conn.sendall(b"ok\n")
        conn.close()
        ret = proc.wait()
        self.assertEqual(ret, 0)

    def test_connection_error(self):
        (out, err, proc) = call_script(self.scriptpath,
                                       ['maven', 'junit:junit'],
                                       extra_env=self.env, async=True)
        conn, _ = self.sock.accept()
        conn.close()
        ret = proc.wait()
        self.assertEqual(ret, 3)

    def test_unsupported_type(self):
        (out, err, ret) = call_script(self.scriptpath,
                                      ['foobar', 'coords'])
        self.assertEqual(ret, 1)
        self.assertEqual(err.rstrip(), "Unsupported artifact type")

    def test_usage(self):
        (out, err, ret) = call_script(self.scriptpath, [])
        self.assertEqual(ret, 1)
        self.assertEqual(err.rstrip(), "Usage: artifact-type artifact-coordinates")

    def tearDown(self):
        try:
            os.unlink(self.sock_path)
        except:
            pass


if __name__ == '__main__':
    unittest.main()



#
# Copyright (c) 2014, Red Hat, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Red Hat nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Authors:  Michal Srb <msrb@redhat.com>

import os
import json
from javapackages.common.exception import JavaPackagesToolsException

# name of the cache file for metadata
metadata_cache_f = "metadata.cache"
# name of the cache file for OSGi stuff
osgi_cache_f = "osgi.cache"


def get_config():
    """Return dictionary representing configuration file,
    None if no valid configuration was found.

    Paths specified in JAVACONFDIRS environment variable are searched for
    configuration file named 'javapackages-config.json'. Only first successfully
    read configuration file is taken into account, rest is ignored. Paths in
    JAVACONFDIRS are expected to be separated by 'os.pathsep'.

    If the JAVACONFDIRS is not defined, '/etc/java/' is assumed.
    """
    if "JAVACONFDIRS" in os.environ:
        config_paths = os.environ["JAVACONFDIRS"].split(os.pathsep)
    else:
        config_paths = ["/etc/java/"]

    for config_path in config_paths:
        try:
            file_path = os.path.join(config_path, "javapackages-config.json")
            with open(file_path) as config_file:
                return json.load(config_file)
        except (OSError, IOError):
            pass

    return None


def get_buildroot():
    """Return buildroot path, raise JavaPackagesToolsException if the path
    couldn't be determined.
    """
    try:
        buildroot = os.environ["RPM_BUILD_ROOT"]
    except KeyError:
        raise JavaPackagesToolsException("RPM_BUILD_ROOT environment "
                                         "variable is not set")
    return os.path.abspath(buildroot)



import unittest
import shutil
import os
from test_common import javautils_script, get_config_file_list, \
        get_actual_config, get_expected_config, \
        preload_xmvn_config, DIRPATH

from xml_compare import compare_xml_files

unordered = ['artifactGlob', 'alias', 'compatVersions']

class TestMvnAlias(unittest.TestCase):

    maxDiff = 2048

    def setUp(self):
        self.olddir = os.getcwd()
        self.workdir = os.path.join(DIRPATH, 'workdir')
        os.mkdir(self.workdir)
        os.chdir(self.workdir)

    def tearDown(self):
        try:
            shutil.rmtree(self.workdir)
        except OSError:
            pass
        os.chdir(self.olddir)

    @javautils_script('mvn_alias', [])
    def test_run_no_args(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertEqual("Usage:", stderr[:6])

    @javautils_script('mvn_alias', ['-h'])
    def test_help(self, stdout, stderr, return_value):
        self.assertTrue(stdout)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy', ])
    def test_simple(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'simple'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy:1.2', ])
    def test_version(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'version'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy:zzz:', ])
    def test_extension(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'extension'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy:zzz:www:', ])
    def test_classifier(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'classifier'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb:3.1', 'xxx:yyy:zzz:3.0', ])
    def test_comb1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'comb1'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb:ccc:ddd:2.1', 'xxx:yyy:', ])
    def test_comb2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'comb2'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb:ccc:', 'xxx:yyy:zzz:www:2.1', ])
    def test_comb3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'comb3'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb:ccc:4.1', 'xxx:yyy:zzz:', ])
    def test_comb4(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'comb4'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa' +
        'aaaaaaaaaaaaaaaaaaaaa:bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb:c' +
        'cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc' +
        'cccccccccc:ddddddddddddddddddddddddddddddddddddddddddddddddddddd' +
        'ddddddddd:1111111111111111111111111111111111111',
        'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx:yyyyyyy' +
        'yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy:zzzzzzzzzzzzzzzzzzzzz' +
        'zzzzzzzzzzzzzzzzzzzzzzzzzzz:wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww' +
        'wwwwwwwwwwwwwwwww:33333333333333333333333333333333333333333333'])
    def test_longopt(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'longopt'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['*:aaa', 'xxx:yyy', ])
    def test_wildcard1(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'wildcard1'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',[':aaa', 'xxx:yyy', ])
    def test_wildcard2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'wildcard2'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa::ccc', 'xxx:yyy', ])
    def test_wildcard3(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'wildcard3'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb', ])
    def test_one_argument(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx', ])
    def test_invalid1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['aaa', 'xxx:yyy', ])
    def test_invalid2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:', ])
    def test_wildcard4(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'wildcard4'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['a:b:c:d:e:f', 'x:y', ])
    def test_invalid5(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['a:b', 'x:y:z:w:1:e', ])
    def test_invalid6(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['aaa:bbb', 'ccc:ddd', 'eee:fff', 'ggg:hhh', ])
    def test_multi(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'multi'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa:bbb', 'ccc:ddd', 'eee:fff', ])
    def test_odd(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'odd'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',[':', ':', ])
    def test_wildcard7(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['x:y', 'a:b:c:*:1', ])
    def test_wildcard8(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['x:y', 'a:b:c::1', ])
    def test_wildcard9(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'wildcard9'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['*:{aaa,bbb}*', ':@1', ])
    def test_backref(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'backref'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',['aaa', ':@1', ])
    def test_backref1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['{aa,bb}:{cc,dd}', '@1:@2', ])
    def test_backref2(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 1)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'backref2'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @javautils_script('mvn_alias',[':{aaa,bbb}', '@1:@2', ])
    def test_backref3(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',[':{aaa,bbb', '@1', ])
    def test_odd_braces1(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',[':{aaa,bbb}}', '@1', ])
    def test_odd_braces2(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    @javautils_script('mvn_alias',['{aaa:bbb}:ccc', '@1', ])
    def test_braces(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertTrue(stderr)

    # for rhbz#1014301
    @javautils_script('mvn_alias',[':{woodstox-core}-asl', '@1-lgpl' ])
    def test_rhbz1014301(self, stdout, stderr, return_value):
        self.assertNotEqual(return_value, 0)
        self.assertEqual(stderr.count("Artifact string '@1-lgpl' does not "
            "contain ':' character. Can not parse"),
                1)

    @preload_xmvn_config('mvn_alias', 'preexisting.xml')
    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy', ])
    def test_preexisting_unindexed(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 2)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'simple'),
                 unordered)
            self.assertFalse(report, '\n' + report)

    @preload_xmvn_config('mvn_alias', 'preexisting.xml',
            dstname='javapackages-config-00001.xml',
        update_index=True)
    @javautils_script('mvn_alias',['aaa:bbb', 'xxx:yyy', ])
    def test_preexisting(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0, stderr)
        filelist = get_config_file_list()
        self.assertEqual(len(filelist), 2)
        for filename in filelist:
            report = compare_xml_files(get_actual_config(filename),
                 get_expected_config(filename, 'mvn_alias', 'preexisting'),
                 unordered)
            self.assertFalse(report, '\n' + report)

if __name__ == '__main__':
    unittest.main()



import os
import unittest

from javapackages.metadata.metadata import Metadata, MetadataInvalidException, MetadataLoadingException
from javapackages.metadata.artifact import MetadataArtifact
from javapackages.metadata.skippedartifact import MetadataSkippedArtifact
from javapackages.metadata.dependency import MetadataDependency

from test.misc import exception_expected


def depmapfile(fname):
    def test_decorator(fn):
        def test_decorated(self, *args, **kwargs):
            main_dir = os.path.dirname(os.path.realpath(__file__))
            fn(self, Metadata.create_from_file(os.path.join(main_dir, "data", fname)))
        return test_decorated
    return test_decorator


class TestDepmap(unittest.TestCase):

    @exception_expected(IOError)
    @depmapfile("NULL_FILE.xml")
    def test_nonexisting_depmap(self, d):
        self.assertTrue(False, "IOError was expected!")

    @depmapfile("depmap_compat_new.xml")
    def test_compat_new(self, d):
        pa = d.get_provided_artifacts()
        self.assertEqual(len(pa), 2)
        for a in pa:
            self.assertTrue(a.is_compat())

    @depmapfile("depmap_compat_new.xml")
    def test_java_requires(self, d):
        self.assertEqual(d.get_java_requires(), ["1.5"])

    @depmapfile("depmap_compat_new.xml")
    def test_java_devel_requires(self, d):
        self.assertEqual(d.get_java_devel_requires(), [])

    @depmapfile("depmap_compat_new.xml")
    def test_single_provides(self, d):
        self.assertEqual(len(d.get_provided_artifacts()), 2)

    @depmapfile("depmap_new_versioned.xml")
    def test_provided_mappings(self, d):
        artifacts = d.get_provided_artifacts()
        self.assertEqual(len(artifacts), 2)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="2.2",
                                         path="/usr/share/java/maven-idea-plugin/maven-idea-plugin.jar",
                                         properties={'requiresJava': '1.5'}
                                         ) in artifacts)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="2.2",
                                         extension="pom",
                                         path="/usr/share/maven-poms/JPP.maven-idea-plugin-maven-idea-plugin.pom")
                        in artifacts)

    @depmapfile("depmap_new_versioned_compressed.xml.gz")
    def test_compressed_depmap(self, d):
        artifacts = d.get_provided_artifacts()
        self.assertEqual(len(artifacts), 2)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="2.2",
                                         path="/usr/share/java/maven-idea-plugin/maven-idea-plugin.jar",
                                         properties={'requiresJava': '1.5'}
                                         ) in artifacts)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="2.2",
                                         path="/usr/share/maven-poms/JPP.maven-idea-plugin-maven-idea-plugin.pom",
                                         extension="pom") in artifacts)

    @depmapfile("depmap_new_compat.xml")
    def test_provided_versioned(self, d):
        artifacts = d.get_provided_artifacts()
        self.assertEqual(len(artifacts), 4)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="1.4",
                                         path="/usr/share/java/maven-idea-plugin/maven-idea-plugin-1.4.jar",
                                         compatVersions=set(['1.4'])) in artifacts)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="1.4",
                                         extension="pom",
                                         path="/usr/share/maven-poms/JPP.maven-idea-plugin-maven-idea-plugin-1.4.pom",
                                         compatVersions=set(["1.4"])) in artifacts)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="1.4",
                                         path="/usr/share/java/maven-idea-plugin/maven-idea-plugin-1.4.jar",
                                         compatVersions=set(["1.5"])) in artifacts)
        self.assertTrue(MetadataArtifact("org.apache.maven.plugins",
                                         "maven-idea-plugin",
                                         version="1.4",
                                         extension="pom",
                                         path="/usr/share/maven-poms/JPP.maven-idea-plugin-maven-idea-plugin-1.4.pom",
                                         compatVersions=set(["1.5"])) in artifacts)

    @exception_expected(MetadataInvalidException)
    @depmapfile("depmap_invalid_nover.xml")
    def test_no_maven_version(self, d):
        d.get_provided_artifacts()

    @exception_expected(MetadataInvalidException)
    @depmapfile("depmap_invalid_nover.xml")
    def test_no_maven_version_mappings(self, d):
        d.get_provided_artifacts()

    @depmapfile("depmap_compat_new.xml")
    def test_multiple_requires(self, d):
        reqs = d.get_required_artifacts()
        self.assertEqual(len(reqs), 8)

        self.assertTrue(MetadataDependency("org.apache.maven",
                                           "maven-project",
                                           requestedVersion="12") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-container-default",
                                           requestedVersion="12") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-utils",
                                           requestedVersion="12",
                                           extension="war") in reqs)

        self.assertTrue(MetadataDependency("org.apache.maven.wagon",
                                           "wagon-provider-api",
                                           requestedVersion="12",
                                           classifier="test-jar") in reqs)

    @depmapfile("depmap_namespace.xml")
    def test_namespace(self, d):
        artifacts = d.get_provided_artifacts()
        for a in artifacts:
            self.assertEqual(a.namespace, "ns")

    @depmapfile("depmap_multiple_namespaces.xml")
    def test_multiple_namespaces(self, d):
        prov = d.get_provided_artifacts()

        self.assertEqual(len(prov), 6)
        self.assertTrue(MetadataArtifact("org.codehaus.plexus",
                                         "plexus-utils",
                                         version="3.0",
                                         namespace="codehaus-plexus",
                                         path="/usr/share/java/plexus-utils/plexus-utils.jar",
                                         properties={'requiresJava': '1.5'}) in prov)

        self.assertTrue(MetadataArtifact("org.codehaus.plexus",
                                         "plexus-utils",
                                         version="3.0",
                                         extension="pom",
                                         namespace="codehaus-plexus",
                                         path="/usr/share/maven-poms/JPP.plexus-utils-plexus-utils.pom",
                                         properties={'requiresJava': '1.5'}) in prov)
        self.assertTrue(MetadataArtifact("plexus",
                                         "plexus-utils",
                                         version="1.2",
                                         namespace="plexus",
                                         path="/usr/share/java/plexus-utils/plexus-utils.jar",
                                         properties={'requiresJava': '1.5'}) in prov)
        self.assertTrue(MetadataArtifact("plexus",
                                         "plexus-utils",
                                         version="1.2",
                                         namespace="plexus",
                                         extension="pom",
                                         path="/usr/share/maven-poms/JPP.plexus-utils-plexus-utils.pom",
                                         properties={'requiresJava': '1.5'}) in prov)
        self.assertTrue(MetadataArtifact("codehaus",
                                         "plexus-utils",
                                         version="1.2",
                                         namespace="codehaus",
                                         path="/usr/share/java/plexus-utils/plexus-utils.jar",
                                         properties={'requiresJava': '1.5'}) in prov)
        self.assertTrue(MetadataArtifact("codehaus",
                                         "plexus-utils",
                                         version="1.2",
                                         namespace="codehaus",
                                         extension="pom",
                                         path="/usr/share/maven-poms/JPP.plexus-utils-plexus-utils.pom",
                                         properties={'requiresJava': '1.5'}) in prov)

    @depmapfile("depmap_namespace_requires.xml")
    def test_requires_namespace(self, d):
        reqs = d.get_required_artifacts()

        self.assertEqual(len(reqs), 12)

        self.assertTrue(MetadataDependency("org.apache.maven",
                                           "maven-project",
                                           requestedVersion="12") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-utils",
                                           requestedVersion="12",
                                           namespace="plexus") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-utils",
                                           requestedVersion="12",
                                           extension="war",
                                           namespace="codehaus") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-utils",
                                           requestedVersion="12",
                                           resolvedVersion="0.9",
                                           namespace="test") in reqs)

        self.assertTrue(MetadataDependency("org.apache.maven.wagon",
                                           "wagon-provider-api",
                                           requestedVersion="12",
                                           classifier="test-jar") in reqs)

        self.assertTrue(MetadataDependency("org.codehaus.plexus",
                                           "plexus-container-default",
                                           requestedVersion="12") in reqs)

    @depmapfile("depmap_skipped.xml")
    def test_skipped(self, d):
        skipped = d.get_skipped_artifacts()

        self.assertEqual(len(skipped), 3)

        self.assertTrue(MetadataSkippedArtifact("org.apache.maven.plugins",
                                                "maven-idea-plugin") in skipped)

        self.assertTrue(MetadataSkippedArtifact("org.apache.maven.plugins",
                                                "maven-idea-plugin",
                                                extension="war") in skipped)

        self.assertTrue(MetadataSkippedArtifact("org.apache.maven.plugins",
                                                "maven-idea-plugin",
                                                classifier="test-jar") in skipped)

    @exception_expected(MetadataLoadingException)
    @depmapfile("depmap_incorrect_provides.xml")
    def test_incorrect_provides(self, d):
        pass

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from lxml.etree import fromstring, parse

from javapackages.maven.artifact import Artifact, ArtifactFormatException

from test.misc import exception_expected

def artifactfile(fname):
    def test_decorator(fn):
        def test_decorated(self, *args, **kwargs):
            main_dir = os.path.dirname(os.path.realpath(__file__))
            fn(self, parse(os.path.join(main_dir, "data", fname)))
        return test_decorated

    return test_decorator

class TestArtifact(unittest.TestCase):

    def setUp(self):
        # intentional whitespaces
        self.gaArtifact = Artifact(' g', ' a')
        self.gavArtifact = Artifact(' g', ' a', ' ', ' ', ' v')
        self.gaeArtifact = Artifact(' g', ' a', 'e')
        self.gacArtifact = Artifact(' g', ' a', ' ', ' c')
        self.gaevArtifact = Artifact(' g', 'a', ' e', ' ', ' v')
        self.gacvArtifact = Artifact(' g', 'a', ' ', 'c', ' v')
        self.fullArtifact = Artifact(' g', ' a', ' e', ' c', ' v')

    def test_artifact_init(self):
        a = self.gaArtifact
        self.assertEqual(a.groupId, 'g')
        self.assertEqual(a.artifactId, 'a')
        self.assertEqual(a.classifier, '')
        self.assertEqual(a.extension, 'jar')
        self.assertEqual(a.version, '')

        a = self.fullArtifact
        self.assertEqual(a.groupId, 'g')
        self.assertEqual(a.artifactId, 'a')
        self.assertEqual(a.classifier, 'c')
        self.assertEqual(a.extension, 'e')
        self.assertEqual(a.version, 'v')

    def test_str(self):
        self.assertEqual(str(self.gaArtifact), "g:a")
        self.assertEqual(str(self.gavArtifact), "g:a:v")
        self.assertEqual(str(self.gacArtifact), "g:a::c:")
        self.assertEqual(str(self.gacvArtifact), "g:a::c:v")
        self.assertEqual(str(self.fullArtifact), "g:a:e:c:v")

    def test_rpm_str(self):
        self.assertEqual(self.gaArtifact.get_rpm_str(), "mvn(g:a)")
        self.assertEqual(self.gavArtifact.get_rpm_str(), "mvn(g:a)")
        self.assertEqual(self.gacArtifact.get_rpm_str(), "mvn(g:a::c:)")
        self.assertEqual(self.gacvArtifact.get_rpm_str(), "mvn(g:a::c:)")
        self.assertEqual(self.gaeArtifact.get_rpm_str(), "mvn(g:a:e:)")
        self.assertEqual(self.gaevArtifact.get_rpm_str(), "mvn(g:a:e:)")
        self.assertEqual(self.fullArtifact.get_rpm_str(), "mvn(g:a:e:c:)")

    def test_xml_str_ga(self):
        doc = fromstring(self.gaArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertEqual(item, None)
        item = doc.find('./classifier')
        self.assertEqual(item, None)
        item = doc.find('./version')
        self.assertEqual(item, None)
        self.assertEqual(len(doc), 2)

    def test_xml_str_gae(self):
        doc = fromstring(self.gaeArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'e')
        item = doc.find('./classifier')
        self.assertEqual(item, None)
        item = doc.find('./version')
        self.assertEqual(item, None)
        self.assertEqual(len(doc), 3)

    def test_xml_str_gac(self):
        doc = fromstring(self.gacArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'jar')
        item = doc.find('./classifier')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'c')
        item = doc.find('./version')
        self.assertEqual(item, None)
        self.assertEqual(len(doc), 4)

    def test_xml_str_gav(self):
        doc = fromstring(self.gavArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'jar')
        item = doc.find('./classifier')
        self.assertEqual(item, None)
        item = doc.find('./version')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'v')
        self.assertEqual(len(doc), 4)

    def test_xml_str_gaev(self):
        doc = fromstring(self.gaevArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'e')
        item = doc.find('./classifier')
        self.assertEqual(item, None)
        item = doc.find('./version')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'v')
        self.assertEqual(len(doc), 4)

    def test_xml_str_gacv(self):
        doc = fromstring(self.gacvArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'jar')
        item = doc.find('./classifier')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'c')
        item = doc.find('./version')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'v')
        self.assertEqual(len(doc), 5)

    def test_xml_str_full(self):
        doc = fromstring(self.fullArtifact.get_xml_str())
        self.assertNotEqual(doc, None)
        item = doc.find('./groupId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'g')
        item = doc.find('./artifactId')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'a')
        item = doc.find('./extension')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'e')
        item = doc.find('./classifier')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'c')
        item = doc.find('./version')
        self.assertNotEqual(item, None)
        self.assertEqual(item.text, 'v')
        self.assertEqual(len(doc), 5)

    @exception_expected(ArtifactFormatException)
    @artifactfile("artifact-empty.xml")
    def test_from_xml_empty(self, doc):
        Artifact.from_xml_element(doc)

    @exception_expected(ArtifactFormatException)
    @artifactfile("artifact-nog.xml")
    def test_from_xml_nog(self, doc):
        Artifact.from_xml_element(doc)

    @exception_expected(ArtifactFormatException)
    @artifactfile("artifact-noa.xml")
    def test_from_xml_noa(self, doc):
        Artifact.from_xml_element(doc)

    @artifactfile("artifactga.xml")
    def test_from_xml_ga(self, doc):
        a = Artifact.from_xml_element(doc)
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

    @artifactfile("artifactgav.xml")
    def test_from_xml_gav(self, doc):
        a = Artifact.from_xml_element(doc)
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v")

    @artifactfile("artifactfull.xml")
    def test_from_xml_full(self, doc):
        a = Artifact.from_xml_element(doc)
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "v")

    @artifactfile("artifact-whitespace.xml")
    def test_from_xml_whitespace(self, doc):
        a = Artifact.from_xml_element(doc)
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "v")

    def test_from_mvn_str_ga(self):
        a = Artifact.from_mvn_str("g:a")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

    def test_from_mvn_str_gav(self):
        a = Artifact.from_mvn_str("g:a:v")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v")

    def test_from_mvn_str_gae(self):
        a = Artifact.from_mvn_str("g:a:e:")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

    def test_from_mvn_str_gaev(self):
        a = Artifact.from_mvn_str("g:a:e:v")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v")

    def test_from_mvn_str_gac(self):
        a = Artifact.from_mvn_str("g:a::c:")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "")

    def test_from_mvn_str_gacv(self):
        a = Artifact.from_mvn_str("g:a::c:v")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "v")

    def test_from_mvn_str_gaec(self):
        a = Artifact.from_mvn_str("g:a:e:c:")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "")

    def test_from_mvn_str_full(self):
        a = Artifact.from_mvn_str("g:a:e:c:v")
        self.assertEqual(a.groupId, "g")
        self.assertEqual(a.artifactId, "a")
        self.assertEqual(a.extension, "e")
        self.assertEqual(a.classifier, "c")
        self.assertEqual(a.version, "v")

    def test_merge1(self):
        a = Artifact.from_mvn_str("g1:a1:v1")
        b = Artifact.from_mvn_str("g1:a1")
        a.merge_with(b)

        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v1")

    def test_merge2(self):
        a = Artifact.from_mvn_str("g1:a1:v1")
        b = Artifact.from_mvn_str("g1:a1:v2")
        a.merge_with(b)

        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v1")

    def test_merge3(self):
        # different artifacts won't be merged
        a = Artifact.from_mvn_str("g1:a1:v1")
        b = Artifact.from_mvn_str("g2:a2:e2::")
        a.merge_with(b)

        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v1")

    def test_merge4(self):
        a = Artifact.from_mvn_str("g1:a1")
        b = Artifact.from_mvn_str("g1:a1:v1")
        a.merge_with(b)

        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "v1")

    def test_merge5(self):
        a = Artifact.from_mvn_str("g1:a1:war::")
        b = Artifact.from_mvn_str("g1:a1:v1")
        a.merge_with(b)

        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "war")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

    def test_interpolation1(self):
        a = Artifact.from_mvn_str("${gid}:a1")
        props = {"gid": "g1"}
        res = a.interpolate(props)

        self.assertEqual(len(res), 0)
        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

    def test_interpolation2(self):
        a = Artifact.from_mvn_str("${gid}:a1:${v.major}.${v.minor}")
        props = {"gid": "g1", "v.major": "4", "v.minor": "11"}
        res = a.interpolate(props)

        self.assertEqual(len(res), 0)
        self.assertEqual(a.groupId, "g1")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "4.11")

    def test_interpolation3(self):
        a = Artifact.from_mvn_str("${gid}:a1")
        props = {"v.minor": "11"}
        res = a.interpolate(props)

        self.assertEqual(len(res), 1)
        self.assertEqual(res[0], "gid")
        self.assertEqual(a.groupId, "${gid}")
        self.assertEqual(a.artifactId, "a1")
        self.assertEqual(a.extension, "jar")
        self.assertEqual(a.classifier, "")
        self.assertEqual(a.version, "")

if __name__ == '__main__':
    unittest.main()



import os
import re
import shutil
import subprocess
import sys
import unittest

from javapackages.metadata.metadata import Metadata
from javapackages.common import util
from os import path
from test_rpmbuild import Package
from xml_compare import compare_xml_files
from functools import wraps
from lxml import etree

DIRPATH = path.dirname(path.realpath(__file__))
SCRIPT_ENV = {
    'PATH': '{mock}:{real}'.format(mock=DIRPATH, real=os.environ['PATH']),
    'LC_ALL': 'C.UTF-8',
}

for var in ('PYTHONPATH', 'COVERAGE_PROCESS_START'):
    if var in os.environ:
        SCRIPT_ENV[var] = os.environ[var]


def call_script(name, args, stdin=None, extra_env={}, async=False):
    with open("tmpout", 'w') as outfile:
        with open("tmperr", 'w') as errfile:
            procargs = [sys.executable, name]
            env = SCRIPT_ENV.copy()
            env.update(extra_env)
            proc = subprocess.Popen(procargs + args, shell=False,
                                    stdout=outfile,
                                    stderr=errfile,
                                    env=env,
                                    stdin=subprocess.PIPE,
                                    universal_newlines=True)
            if async:
                return (outfile, errfile, proc)
            proc.communicate(stdin)
            ret = proc.wait()
    with open("tmpout", 'r') as outfile:
        out = outfile.read()
    with open("tmperr", 'r') as errfile:
        err = errfile.read()
    os.remove('tmpout')
    os.remove('tmperr')
    return (out, err, ret)


def get_config_file_list():
    try:
        return os.listdir('.xmvn/config.d/')
    except OSError:
        return []

def get_actual_config(filename):
    return path.join('.xmvn', 'config.d', filename)

def get_expected_config(filename, scriptname, testname):
    fileno = re.findall('[0-9]+', filename)
    if fileno:
        expfname = '{name}_{idx}.xml'.format(name=testname, idx=fileno[-1])
    else:
        expfname = filename
    return path.join(DIRPATH, 'data', scriptname, expfname)


def get_actual_args():
    with open('.xmvn/out', 'r') as f:
        args = f.read()
    return args


def get_expected_args(scriptname, testname):
    fpath = path.join(DIRPATH, 'data', scriptname, "{name}_out".format(name=testname))
    with open(fpath, 'r') as f:
        args = f.read()
    return args


def preload_xmvn_config(name, filename, dstname=None, update_index=False):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            src = path.join(DIRPATH, 'data', name, filename)
            os.mkdir('.xmvn')
            os.mkdir('.xmvn/config.d')
            dst = path.join('.xmvn', 'config.d', dstname or filename)
            shutil.copy(src, dst)
            if update_index:
                idx = 1
                if path.exists('.xmvn/javapackages-rule-index'):
                    with open('.xmvn/javapackages-rule-index', 'r') as index:
                        idx = int(index.read())
                with open('.xmvn/javapackages-rule-index', 'w') as index:
                    index.write(str(idx))
            fun(self)
        return test_decorated
    return test_decorator


def prepare_metadata(metadata_dir):
    for dirname, dirnames, filenames in os.walk(metadata_dir):
        for filename in filenames:
            if filename.endswith("-want.xml"):
                want_file = os.path.join(dirname, filename)
                xml = etree.parse(want_file)
                for path in xml.xpath('//*[local-name()="path"]'):
                    if '%' in path.text:
                        path.text = path.text % (metadata_dir)
                xml.write(want_file)


def javautils_script(name, fnargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            scriptpath = path.join(DIRPATH, '..', 'java-utils', name + '.py')
            (stdout, stderr, return_value) = call_script(scriptpath, fnargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def bindir_script(name, fnargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            scriptpath = path.join(DIRPATH, '..', 'bin', name)
            (stdout, stderr, return_value) = call_script(scriptpath, fnargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def call_rpmgen(rpmgen_name, filelist_prefix, filelist, env=None,
                config=''):
    scriptpath = path.join(DIRPATH, '..', 'depgenerators', rpmgen_name)
    stdin = []
    if not filelist:
        filelist = ["/dev/null"]

    if filelist_prefix:
        stdin.extend([os.path.abspath(os.path.join(filelist_prefix, x))
                     for x in filelist])
    else:
        stdin = filelist

    if not env:
        env = {}

    buildroot = "/dev/null"
    if "RPM_BUILD_ROOT" not in env and stdin:
        result = re.match(".*?/buildroot/", stdin[0])
        if result:
            buildroot = os.path.abspath(result.group(0))
    env.update({"RPM_BUILD_ROOT": buildroot})

    try:
        shutil.rmtree("/tmp/.javapackages_cache/")
    except OSError:
        pass
    for line in stdin:
        # FIXME this PID is a temporary hack
        ret = call_script(scriptpath, ["--cachedir", "/tmp", "--rpm-pid", "1"],
                          stdin=line, extra_env=env)
    try:
        shutil.rmtree("/tmp/.javapackages_cache/")
    except OSError:
        pass
    return ret


def osgiprov(*args, **kwargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            (stdout, stderr, return_value) = call_rpmgen("osgi.prov",
                                                         "data/osgi/",
                                                         *args, **kwargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def osgireq(*args, **kwargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            (stdout, stderr, return_value) = call_rpmgen("osgi.req",
                                                         "data/osgi/",
                                                         *args, **kwargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def mavenprov(*args, **kwargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            (stdout, stderr, return_value) = call_rpmgen("maven.prov",
                                                         "metadata/",
                                                         *args, **kwargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def mavenreq(*args, **kwargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            pargs, pkwargs = rpmgen_process_args(args, kwargs)
            (stdout, stderr, return_value) = call_rpmgen("maven.req",
                                                         "metadata/",
                                                         *pargs,
                                                         **pkwargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def javadocreq(*args, **kwargs):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            pargs, pkwargs = rpmgen_process_args(args, kwargs)
            (stdout, stderr, return_value) = call_rpmgen("javadoc.req",
                                                         "",
                                                         *pargs,
                                                         **pkwargs)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator


def rpmgen_process_args(args, kwargs):
    if "javaconfdirs" in kwargs:
        confdirs = [os.path.join(DIRPATH, "data", "config", conf) for conf in kwargs["javaconfdirs"]]
        try:
            env = kwargs["env"]
        except KeyError:
            env = {}
        env.update({"JAVACONFDIRS": os.pathsep.join(confdirs)})
        kwargs.update({"env": env})
        del kwargs["javaconfdirs"]
    else:
        env = {}
        env.update({"JAVACONFDIRS": os.path.abspath(os.path.join(DIRPATH, '..', 'etc'))})
        kwargs.update({"env": env})

    if "xmvnresolve_output" in kwargs:
        dest = "/tmp/"
        src = os.path.abspath(os.path.join("resolve", kwargs["xmvnresolve_output"]))
        shutil.copy(src, dest)
        try:
            env = kwargs["env"]
        except KeyError:
            env = {}
        env.update({"JAVAPACKAGES_XMVN_RESOLVE_TEST": os.path.join(dest, os.path.basename(src))})
        kwargs.update({"env": env})
        del kwargs["xmvnresolve_output"]
    return args, kwargs


def mvn_depmap(pom, jar=None, fnargs=None):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            os.chdir(self.workdir)
            buildroot = os.path.join(self.workdir, "builddir/build/BUILDROOT")
            env = {'RPM_BUILD_ROOT': buildroot}
            scriptpath = path.join(DIRPATH, '..', 'java-utils', 'maven_depmap.py')
            args = ['.fragment_data', pom]
            if jar:
                args.append(path.join(os.getcwd(), jar))
            args.extend(fnargs or [])
            (stdout, stderr, return_value) = call_script(scriptpath, args, extra_env=env)
            frag = None
            if return_value == 0:
                with open('.fragment_data','r') as frag_file:
                    frag = frag_file.read()
                os.remove('.fragment_data')
            fun(self, stdout, stderr, return_value, depmap=frag)
        return test_decorated
    return test_decorator

def mvn_artifact(pom, jar=None):
    def test_decorator(fun):
        @wraps(fun)
        def test_decorated(self):
            os.chdir(self.datadir)
            scriptpath = path.join(DIRPATH, '..', 'java-utils', 'mvn_artifact.py')
            os.chdir(self.workdir)
            args = [pom]
            if jar:
                args.append(path.join(os.getcwd(), jar))
            (stdout, stderr, return_value) = call_script(scriptpath, args)
            fun(self, stdout, stderr, return_value)
        return test_decorated
    return test_decorator

class WorkdirTestCase(unittest.TestCase):
    olddir = os.getcwd()
    WORKDIR = '.workdir'

    def setUp(self):
        self.olddir = os.getcwd()
        try:
            shutil.rmtree(self.WORKDIR)
        except OSError:
            pass
        os.mkdir(self.WORKDIR)
        os.chdir(self.WORKDIR)

    def tearDown(self):
        try:
            shutil.rmtree(self.WORKDIR)
        except OSError:
            pass
        os.chdir(self.olddir)

def exec_pom_macro(line, poms_tree, want_tree=None, filename='pom.xml'):
    """
    Parameters:
        line::
            A line of spec code injected to %prep
        poms_tree::
            dictionary that maps subpackage directory paths to input poms
        want_tree::
            dictionary that maps subpackage directory paths to wanted poms
            (in want directory)
    It creates a directory structure corresponding to keys in poms_tree and
    copies pom files into it. Then it executes prep and compares altered poms
    to wanted ones from want_tree (if not specified in want_tree it is assumed
    to remain unchanged and is compared with the original pom). Returns tuple
    of rpmbuild's return value, stderr and report of differences in xml files.
    """
    DATADIR = path.join(DIRPATH, 'data', 'pom_editor')
    pack = Package('test')
    pack.append_to_prep(line)
    for destpath, sourcepath in poms_tree.items():
        pack.add_source(path.join(DATADIR, sourcepath), path.join(destpath, filename))
    _, stderr, return_value = pack.run_prep()
    reports = []
    if return_value == 0:
        for filepath, pom in poms_tree.items():
            if want_tree and filepath in want_tree:
                expected_pom = path.join('want', want_tree[filepath])
            else:
                expected_pom = pom
            expected_pom = path.join(DATADIR, expected_pom)
            actual_pom = path.join(pack.buildpath, filepath, filename)
            reports.append(compare_xml_files(actual_pom, expected_pom))
    return return_value, stderr, '\n'.join(reports).strip()

def exec_pom_macro_simple(line, pom, want=None, filename='pom.xml'):
    return exec_pom_macro(line, {'': pom}, {'': want} if want else None, filename=filename)


def assertIn(obj, item, iterable):
    obj.assertTrue(item in iterable, msg="{item} not found in {iterable}"
                   .format(item=item, iterable=iterable))



import os
import unittest
import lxml

from javapackages.maven.pom import POM, PomLoadingException

from test.misc import exception_expected


def pomfile(fname):
    def test_decorator(fn):
        def test_decorated(self, *args, **kwargs):
            main_dir = os.path.dirname(os.path.realpath(__file__))
            fn(self, POM(os.path.join(main_dir, "data", fname)))
        return test_decorated

    return test_decorator


class TestPOM(unittest.TestCase):

    @exception_expected(PomLoadingException)
    @pomfile("NULL_FILE.pom.xml")
    def test_nonexisting_pom(self, p):
        self.assertTrue(False, "IOError was expected!")

    @pomfile("commons-lang.pom")
    def test_loading(self, p):
        self.assertTrue(True)

    @pomfile("commons-lang.pom")
    def test_with_parent(self, p):
        self.assertEqual(p.artifactId, "commons-lang")
        self.assertEqual(p.groupId, "commons-lang")
        self.assertEqual(p.version, "2.6")
        self.assertNotEqual(p.artifactId, "commons-parent")
        self.assertNotEqual(p.groupId, "org.apache.commons")
        self.assertNotEqual(p.version, "17")
        self.assertEqual(p.packaging, "jar")

    @pomfile("xmlrpc.pom")
    def test_parent_pom(self, p):
        self.assertEqual(p.packaging, "pom")
        self.assertEqual(p.groupId, "org.apache.xmlrpc")
        self.assertEqual(p.artifactId, "xmlrpc")
        self.assertEqual(p.version, "3.1.3")

    @pomfile("xmlrpc-nons.pom")
    def test_no_xmlns(self, p):
        self.assertEqual(p.packaging, "pom")
        self.assertEqual(p.groupId, "org.apache.xmlrpc")
        self.assertEqual(p.artifactId, "xmlrpc")
        self.assertEqual(p.version, "3.1.3")

    @pomfile("parent-version.pom")
    def test_parent_version(self, p):
        self.assertEqual(p.packaging, "jar")
        self.assertEqual(p.groupId, "commons-lang")
        self.assertEqual(p.artifactId, "commons-lang")
        self.assertEqual(p.version, "17")

    @exception_expected(lxml.etree.XMLSyntaxError)
    @pomfile("unparsable_xml.pom")
    def test_unparsable_xml(self, p):
        self.fail("Unparsable xml successfully parsed")

    @pomfile("junit-comments.pom")
    def test_pom_comments(self, p):
        self.assertEqual(p.packaging, "jar")
        self.assertEqual(p.groupId, "junit")
        self.assertEqual(p.artifactId, "junit")
        self.assertEqual(p.version, "4.11")

    @exception_expected(PomLoadingException)
    @pomfile("ivy-simple.xml")
    def test_ivy_module(self, p):
        self.assertEqual(p.groupId, "org.apache")


if __name__ == '__main__':
    unittest.main()



import unittest
from test_common import javautils_script


class TestBuilddep(unittest.TestCase):

    cmd = "builddep"
    p = "../test/builddep/"

    @javautils_script(cmd, [p + "compat"])
    def test_compat(self, stdout, stderr, return_value):
        self.assertEqual(return_value, 0)
        sout = [x for x in stdout.split('\n') if x]
        want = ("mvn(org.codehaus.groovy:groovy:1.8.9)",)
        self.assertEqual(set(want), set(sout))


if __name__ == '__main__':
    unittest.main()



import os
from setuptools import setup, find_packages

dirpath = os.path.dirname(os.path.realpath(__file__))
versionpath = os.path.join(dirpath, "..", "VERSION")
moduleversion = os.path.join(dirpath, "javapackages", "version.py")
with open(versionpath) as vp:
    __version__ = vp.read().strip()
    with open(moduleversion, "w") as mv:
        mv.write("__version__ = '{v}'".format(v=__version__))

setup(
    name="javapackages",
    description="Module for handling, querying and manipulating of various "
                "files for Java packaging in Linux distributions",
    version=__version__,
    license="BSD",
    download_url="https://github.com/fedora-java/javapackages/releases",
    url="https://github.com/fedora-java/javapackages",
    packages=find_packages(exclude=["test"]),
    test_suite="test",
    maintainer="javapackages maintainers",
    maintainer_email="javapackages-tools-owner@fedoraproject.org"
)
